#ifndef GLOBAL_H
#define GLOBAL_H
/*
* Dateiname.... global.h
* erstellt am.. 2016-10-30
* gendert am.. 2017-05-01
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... based on RFM_wireless_4CH_sensor_02_mod
* .............	modify radio parameters, change to 30kHz frequency deviation
* .............	output power to 0dBm, alter data rate to net 26.3kbaud
* ............. revise RMF settings: low batt detector command needs to enable 
* ............. the 30sec timer calibration
* ............. change array declaration of "ADC_previous_value[]"
* ............. simplify the define-settings for RFM operation
* ............. implementation of water level sensing, add defines for ADC channels
*/

// global includes
//------------------------------------------------------------------
#include <avr/io.h>
#include <inttypes.h>
#include "asm.h"

// global defines
//------------------------------------------------------------------
#define ON 1
#define OFF 0
#define OUT 1
#define IN 0
#define HIGH 1
#define LOW 0
#define TRUE 1
#define FALSE 0

/* number of sensors active (compile switch)
* - BATT is 2byte battery voltage as ADC raw LSB, MSB => always on! cannot be disabled
* - S_TEMP is 2byte integrated temperature sensor as ADC raw LSB, MSB
* - S_ILLU is 2byte illumination as ADC raw LSB, MSB
* - S_TEMP_EXT1 is 2byte external temperature sensor 1 as ADC raw LSB, MSB
* - S_TEMP_EXT2 is 2byte external temperature sensor 2 as ADC raw LSB, MSB
* - S_WATER3CH is a 2byte water level value composed of 3 ADC values (ADC0 ADC1 and ADC5)
*              the sensor is made of 4 rods where one rod connects to ground and the other three
*              are forming a terminal input to a parallel resistor to ground each
*/
// comment or uncomment for applicability
#define S_TEMP 0x80
//#define S_ILLU 0x20
//#define S_TEMP_EXT1 0x08
//#define S_TEMP_EXT2 0x02
#define S_WATER3CH 0x40

/* hardware pin assignment between RFM02 and MCU
* FSK <--- PB0 = 1 
* SEL <--- PB2 = 3
* IRQ ---> PB6 = 9
* SDI <--- PA7 = 11
* SCK <--- PA6 = 12 
*/
// uncomment the next define if design has no external clock source, check settings of fuse bits
// comment the next define if design has an external clock source
#define NOCRYSTAL

// to enable some debugging features uncomment the next define
//#define DEBUGON 1
//#define DEBUGADCON 1
//#define DEBUGADCTHRESHOLD 1
//#define ADCTHRESHOLDINACTIVE 1

#define RFM_FSK_PORT PORTB
#define RFM_FSK_POUT PORTB0
#define RFM_FSK_DDR (DDRB |= (0x01 << RFM_FSK_POUT))
#define RFM_FSK(x) do { if (x==HIGH) {RFM_FSK_PORT |= (0x01 << RFM_FSK_POUT);} else {RFM_FSK_PORT &= ~(0x01 << RFM_FSK_POUT);}} while(0)

#define RFM_SEL_PORT PORTB
#define RFM_SEL_POUT PORTB2
#define RFM_SEL_DDR (DDRB |= (0x01 << RFM_SEL_POUT))
#define RFM_SEL(x) do { if (x==HIGH) {RFM_SEL_PORT |= (0x01 << RFM_SEL_POUT);} else {RFM_SEL_PORT &= ~(0x01 << RFM_SEL_POUT);}} while(0)

#define RFM_IRQ_PORT PORTB
#define RFM_IRQ_PIN PINB
#define RFM_IRQ_POUT PORTB6
#define RFM_IRQ_DDR (DDRB &= ~(0x01 << RFM_IRQ_POUT))
#define RFM_IRQ_HIGH (RFM_IRQ_PIN & (0x01 << RFM_IRQ_POUT))

#define RFM_SDI_PORT PORTA
#define RFM_SDI_POUT PORTA7
#define RFM_SDI_DDR (DDRA |= (0x01 << RFM_SDI_POUT))
#define RFM_SDI(x) do { if (x==HIGH) {RFM_SDI_PORT |= (0x01 << RFM_SDI_POUT);} else {RFM_SDI_PORT &= ~(0x01 << RFM_SDI_POUT);}} while(0)

#define RFM_SCK_PORT PORTA
#define RFM_SCK_POUT PORTA6
#define RFM_SCK_DDR (DDRA |= (0x01 << RFM_SCK_POUT))
#define RFM_SCK(x) do { if (x==HIGH) {RFM_SCK_PORT |= (0x01 << RFM_SCK_POUT);} else {RFM_SCK_PORT &= ~(0x01 << RFM_SCK_POUT);}} while(0)

#define LED1_PORT PORTB // this is the flash LED to indicate RF-transmission actions
#define LED1_POUT PORTB1
#define LED1_DDR (DDRB |= (0x01 << LED1_POUT))
#define LED1(x) do { if (x==ON) {LED1_PORT |= (0x01 << LED1_POUT);} else {LED1_PORT &= ~(0x01 << LED1_POUT);}} while(0)

#define DEBUG1_PORT PORTB
#define DEBUG1_POUT PORTB1 // use the LED port, don't use debug port while flashing the LED or disable LED
#define DEBUG1_DDR (DDRB |= (0x01 << DEBUG1_POUT))
#define DEBUG1(x) do { if (x==HIGH) {DEBUG1_PORT |= (0x01 << DEBUG1_POUT);} else {DEBUG1_PORT &= ~(0x01 << DEBUG1_POUT);}} while(0)

// ---- settings for ADC measurements ------
#define SENS_EN_PORT PORTB
#define SENS_EN_POUT PORTB3
#define SENS_EN_DDR (DDRB |= (0x01 << SENS_EN_POUT))
#define SENS_EN(x) do { if (x==ON) {SENS_EN_PORT |= (0x01 << SENS_EN_POUT);} else {SENS_EN_PORT &= ~(0x01 << SENS_EN_POUT);}} while(0)

#define ILLUM_SENS_PORT PORTA
#define ILLUM_SENS_PIN PINA
#define ILLUM_SENS_POUT PORTA5
#define ILLUM_SENS_MUX 4

#define BATT_SENS_PORT PORTA
#define BATT_SENS_PIN PINA
#define BATT_SENS_POUT PORTA4
#define BATT_SENS_MUX 3

#define EXT_SENS2_PORT PORTA
#define EXT_SENS2_PIN PINA
#define EXT_SENS2_POUT PORTA2
#define EXT_SENS2_MUX 2

#define EXT_SENS1_PORT PORTA
#define EXT_SENS1_PIN PINA
#define EXT_SENS1_POUT PORTA1
#define EXT_SENS1_MUX 1

#define TEMP_SENS_PORT PORTA
#define TEMP_SENS_PIN PINA
#define TEMP_SENS_POUT PORTA0
#define TEMP_SENS_MUX 0

#define WATERLEVELCH1_SENS_PORT PORTA
#define WATERLEVELCH1_SENS_PIN PINA
#define WATERLEVELCH1_SENS_POUT PORTA1
#define WATERLEVELCH1_SENS_MUX 1

#define WATERLEVELCH2_SENS_PORT PORTA
#define WATERLEVELCH2_SENS_PIN PINA
#define WATERLEVELCH2_SENS_POUT PORTA2
#define WATERLEVELCH2_SENS_MUX 2

#define WATERLEVELCH3_SENS_PORT PORTA
#define WATERLEVELCH3_SENS_PIN PINA
#define WATERLEVELCH3_SENS_POUT PORTA5
#define WATERLEVELCH3_SENS_MUX 4
// ---- end ADC settings -------------------

// ---- settings for the RFM02 -------------
#define ENABLE_CRYSTAL_EX 5
#define ENABLE_SYNTHESIZER_ES 4
#define ENABLE_AMPLIFIER_EA 3
#define ENABLE_LOW_BATT_DETECT_EB 2
#define ENABLE_WAKEUP_TIMER_ET 1
#define DISABLE_TIMER_CALIBRATION_DWC 7
#define ENABLE_TX_BIT_SYNCHRO_EBS 5

//#define RFM_CMD_CONFIGSET 0x9061        // frequency deviation 60kHz, 11.5pF crystal load
#define RFM_CMD_CONFIGSET 0x9060        // frequency deviation 30kHz, 11.5pF crystal load
#define RFM_CMD_FREQSET 0xA67C			// 868.3MHz
#define RFM_CMD_PWRSET 0xB100			// -3dB, FSK
//#define RFM_CMD_PWRSET 0xB000			// 0dB, FSK
#define RFM_CMD_PLLSET 0xD2C0           // up to 38.4kBaud, -110dBc/Hz
//#define RFM_CMD_DATARATE 0xC811         // this is ~19.2kbaud, with the half data rate command => 9.6kbaud
#define RFM_CMD_DATARATE 0xC806         // this is ~49.3kbaud, with the half data rate command => 24.6kbaud

#define RFM_CMD_LBDSET 0xC200		// default: TX bit synchronisation disabled, 30 sec timer calibration enabled

#define RFM_CMD_PWRMNG 0xC000		// default: all off

#define RFM_CMD_SLEEP 0xC410			// the sleep command, wait 16 clock cycles before sleep

#define RFM_CMD_HALFDATARATE 0xD040
#define RFM_CMD_READSTATUS 0xCC00

#define REPEAT_SLEEP 0xE909			// equals about 4.6 seconds
//#define LONG_SLEEP 0xE7EA			// equals about 30 seconds
#define LONG_SLEEP 0xE9B0			// equals about 90 seconds
// ---- end RFM02 setting ------------------

#define MAX_BUFFER_SIZE 35
#define MAX_ADC_BUFFER 5       // maximum 5 channels with 2byte per channel
#define MAX_TX_HOLDOFF 15       // maximum number of cycles without TX activity if no sensor value changes

// ---- status_flag type map ---------------
#define FIRST_RF_TX 0x0001
#define REPEAT_RF_TX 0x0004
#define REPEAT_INIT_ALL 0x1000
#define REPEAT_ACTIVE FALSE

// global variables
//------------------------------------------------------------------
volatile uint8_t TX_buffer[MAX_BUFFER_SIZE];
volatile uint16_t ADC_previous_value[MAX_ADC_BUFFER];
volatile uint8_t ADC_TX_holdoff[MAX_ADC_BUFFER]; // carries a boolean flag
volatile uint8_t telegram_length;
volatile uint16_t status_flag;
volatile uint8_t ADC_change; // carries a boolean flag
volatile uint8_t global_TX_holdoff;

#endif